/**
 * $Id: $
 *
 * @brief Red Pitaya Applications library common module interface
 *
 * @Author Luka Golinar, RedPitaya
 *
 * (c) Red Pitaya  http://www.redpitaya.com
 *
 * This part of code is written in C programming language.
 * Please visit http://en.wikipedia.org/wiki/C_(programming_language)
 * for more details on the language used herein.
 */

 #ifndef UTILS_H_
 #define UTILS_H_


#include <sys/syslog.h>
#include "rp.h"

#define RP_LOG(...){ \
	syslog(__VA_ARGS__); \
}

#define MAX(x, y) (((x) > (y)) ? (x) : (y))
#define MIN(x, y) (((x) < (y)) ? (x) : (y))
#define ABS(x) (((x) < 0) ? (-(x)) : (x))

#define EXEC_CHECK(x){ \
 		int retval = (x); \
 		if(retval != RP_OK) { \
 			RP_LOG(LOG_INFO, "Execution error.\n"); \
 			return retval; \
 		} \
}

#define ECHECK_APP_MUTEX(MUTEX, x) { \
    int retval = (x); \
    if (retval != RP_OK) { \
        RP_LOG(LOG_INFO, "Execution error.\n");  \
        pthread_mutex_unlock(&MUTEX); \
        return retval; \
    } \
}

#define ECHECK(x) { \
    int retval = (x); \
    if (retval != RP_OK) { \
    fprintf(stderr, "Runtime error: %s returned \"%s\" at %s:%d\n", #x, rp_GetError(retval), __FILE__, __LINE__); \
    return retval; \
    } \
    }


float vectorMax(float *data, int size);
float vectorApprox(float *data, int size, float approx_val, bool min);
float trapezoidalApprox(double *data, float t, int size);
void FirRectangleWindow(float *data, int data_size, int window_size);
float vectorMean(float *data, int steps);
float **multiDimensionVector(int second_dimenson);
int delMultiDimensionVector(float** data);
int set_IIC_Shunt(int k);
bool isSineTester(float **data, uint32_t size, double T);

void lcr_getDecimationValue(float frequency,
						rp_acq_decimation_t *api_dec,
						int *dec_val);
int checkExtensionModuleConnection();

#endif //UTILS_H_
